import { NestInterceptor, Type } from '@nestjs/common';
import { MongestService } from 'mongest-service';
import { ObjectId } from 'mongodb';
import { EntityPayload, ItemOrArray, MongoDoc, NestGuardClassOrInstance } from 'src/types';
import { CustomFilter, FilterBuilder } from './CustomFilter';
import { RaPaginationArgs } from './pagination';
interface ArgsOptions {
    omitFields?: string[];
}
interface ResolverEndpointDefaultOptions {
    guard?: ItemOrArray<NestGuardClassOrInstance>;
    interceptor?: ItemOrArray<NestInterceptor>;
    enable?: boolean;
}
interface VirtualFieldOptions<T> {
    dependsOn?: (string & keyof T)[];
}
export interface MongestRaResolverOptions<T extends EntityPayload, F extends CustomFilter = {}, IdType = ObjectId> {
    filter?: {
        classRef: Type<F>;
        filterBuilder: FilterBuilder<MongoDoc<T, IdType>, F>;
    };
    virtualFields?: Record<string, VirtualFieldOptions<MongoDoc<T, IdType>>>;
    discriminatorRequiredExtraFields?: (string & keyof MongoDoc<T, IdType>)[];
    endpoints?: {
        getOne?: ResolverEndpointDefaultOptions;
        getMany?: ResolverEndpointDefaultOptions;
        create?: ResolverEndpointDefaultOptions & {
            args?: Type<EntityPayload> | ArgsOptions;
        };
        update?: ResolverEndpointDefaultOptions & {
            args?: Type<{
                id: unknown;
            } & EntityPayload> | ArgsOptions;
        };
        delete?: ResolverEndpointDefaultOptions;
    };
}
export declare type GetManyArgs<T extends EntityPayload, F extends object | undefined = undefined> = RaPaginationArgs<T> & {
    filter?: F;
};
export declare function BuildGetManyArgs<T extends MongoDoc<EntityPayload, any>, F extends CustomFilter>(filterOptions: MongestRaResolverOptions<T, F>['filter'] | undefined): Type<GetManyArgs<T, F>>;
export declare function BuildMongestRaResolver<T extends EntityPayload, F extends CustomFilter, Service extends MongestService<T, IdType>, IdType = ObjectId>(entity: Type<T>, options?: MongestRaResolverOptions<T, F, IdType>): Type<T & {
    service: Service;
}>;
export {};
